export * from './exports';
import { getBatch } from './utils/batch'; // For other renderers besides ReactDOM and React Native,
// use the default noop batch function

var batch = getBatch();
export { batch };