"use strict";

exports.__esModule = true;
exports.useIsomorphicLayoutEffect = void 0;

var _react = require("react");

// Under React Native, we know that we always want to use useLayoutEffect
var useIsomorphicLayoutEffect = _react.useLayoutEffect;
exports.useIsomorphicLayoutEffect = useIsomorphicLayoutEffect;