(window["anomalyDetectionDashboards_bundle_jsonpfunction"]=window["anomalyDetectionDashboards_bundle_jsonpfunction"]||[]).push([[25],{2809:function(module,exports,__webpack_require__){"use strict";var _interopRequireWildcard=__webpack_require__(9).default;Object.defineProperty(exports,"__esModule",{value:true});exports.default=void 0;var _react=_interopRequireWildcard(__webpack_require__(2));var _lodash=__webpack_require__(8);var _i18n=__webpack_require__(4);var _reactUse=__webpack_require__(153);var _objects_table=__webpack_require__(819);var _utils=__webpack_require__(371);const SavedObjectsTablePage=_ref=>{let{coreStart:coreStart,dataStart:dataStart,allowedTypes:allowedTypes,serviceRegistry:serviceRegistry,actionRegistry:actionRegistry,columnRegistry:columnRegistry,namespaceRegistry:namespaceRegistry,setBreadcrumbs:setBreadcrumbs,dataSourceEnabled:dataSourceEnabled,dataSourceManagement:dataSourceManagement,navigation:navigation,useUpdatedUX:useUpdatedUX}=_ref;const capabilities=coreStart.application.capabilities;const itemsPerPage=coreStart.uiSettings.get("savedObjects:perPage",50);const dateFormat=coreStart.uiSettings.get("dateFormat");const currentWorkspace=(0,_reactUse.useObservable)(coreStart.workspaces.currentWorkspace$);(0,_react.useEffect)(()=>{setBreadcrumbs([useUpdatedUX?currentWorkspace?{text:_i18n.i18n.translate("savedObjectsManagement.updatedUX.workspace.title",{defaultMessage:"Workspace assets"})}:{text:_i18n.i18n.translate("savedObjectsManagement.updatedUX.title",{defaultMessage:"Assets"})}:{text:_i18n.i18n.translate("savedObjectsManagement.breadcrumb.index",{defaultMessage:"Saved objects"}),href:"/"}])},[setBreadcrumbs,useUpdatedUX,currentWorkspace]);return _react.default.createElement(_objects_table.SavedObjectsTable,{allowedTypes:allowedTypes,serviceRegistry:serviceRegistry,actionRegistry:actionRegistry,columnRegistry:columnRegistry,namespaceRegistry:namespaceRegistry,savedObjectsClient:coreStart.savedObjects.client,indexPatterns:dataStart.indexPatterns,search:dataStart.search,http:coreStart.http,overlays:coreStart.overlays,notifications:coreStart.notifications,applications:coreStart.application,workspaces:coreStart.workspaces,perPageConfig:itemsPerPage,goInspectObject:savedObject=>{const inAppUrl=(0,_utils.formatInspectUrl)(savedObject,coreStart);if(inAppUrl){return coreStart.application.navigateToUrl(inAppUrl)}},dateFormat:dateFormat,canGoInApp:savedObject=>{const{inAppUrl:inAppUrl}=savedObject.meta;return inAppUrl?Boolean((0,_lodash.get)(capabilities,inAppUrl.uiCapabilitiesPath)):false},dataSourceEnabled:dataSourceEnabled,dataSourceManagement:dataSourceManagement,navigationUI:navigation.ui,useUpdatedUX:useUpdatedUX})};exports.default=SavedObjectsTablePage;module.exports=exports.default}}]);