(window["anomalyDetectionDashboards_bundle_jsonpfunction"]=window["anomalyDetectionDashboards_bundle_jsonpfunction"]||[]).push([[24],{2799:function(module,exports,__webpack_require__){"use strict";var _interopRequireDefault=__webpack_require__(0).default;var _interopRequireWildcard=__webpack_require__(9).default;Object.defineProperty(exports,"__esModule",{value:true});exports.default=exports.MarkdownSimple=void 0;var _react=_interopRequireWildcard(__webpack_require__(2));var _reactMarkdown=_interopRequireDefault(__webpack_require__(2791));const markdownRenderers={root:_react.Fragment};const MarkdownSimple=_ref=>{let{children:children}=_ref;return _react.default.createElement(_reactMarkdown.default,{renderers:markdownRenderers},children)};exports.MarkdownSimple=MarkdownSimple;var _default=exports.default=MarkdownSimple}}]);